package fr.natan.microservices.springsecurityserverjwt.service.services;

import fr.natan.microservices.springsecurityserverjwt.models.AppRole;
import fr.natan.microservices.springsecurityserverjwt.models.UserAccount;
import fr.natan.microservices.springsecurityserverjwt.models.UserAccountRoleForm;
import fr.natan.microservices.springsecurityserverjwt.moduleExceptions.businessExceptions.*;
import fr.natan.microservices.springsecurityserverjwt.repositories.RoleRepo;
import fr.natan.microservices.springsecurityserverjwt.repositories.UserAccountRepo;
import fr.natan.microservices.springsecurityserverjwt.service.utilities.UserAccountAndRolesValidator;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.Collection;

@Service
@Transactional
public class AccountServiceImpl implements AccountService {
    private final UserAccountRepo userAccountRepo;
    private final RoleRepo roleRepo;
    private final PasswordEncoder passwordEncoder;

    public AccountServiceImpl(UserAccountRepo userAccountRepo, RoleRepo roleRepo,
                              PasswordEncoder passwordEncoder) {
        this.userAccountRepo = userAccountRepo;
        this.roleRepo = roleRepo;
        this.passwordEncoder = passwordEncoder;
    }

    @Override
    public UserAccount addNewUserAccount(UserAccount userAccount) throws UserAccountAlreadyExistsException,
            UserAccountFieldsInvalidException {

        if (!UserAccountAndRolesValidator.isValidUserAccount(userAccount)) {
            throw new UserAccountFieldsInvalidException();
        }
        UserAccount formattedUserAccount = UserAccountAndRolesValidator.formatUserAccount(userAccount);

        if (getUserByUsername(formattedUserAccount.getUsername())!=null) {
            throw new UserAccountAlreadyExistsException();
        }

        String password = formattedUserAccount.getPwd();
        formattedUserAccount.setPwd(passwordEncoder.encode(password));


        return userAccountRepo.save(formattedUserAccount);
    }

    @Override
    public UserAccount getUserByUsername(String username) {
       UserAccount userAccount =  userAccountRepo.findByUsername(username);

        return userAccount;
    }

    @Override
    public Collection<UserAccount> loadAllUsers() {
        return userAccountRepo.findAll();
    }

    @Override
    public Collection<AppRole> loadAllRoles() {
        return roleRepo.findAll();
    }

    @Override
    public AppRole addNewRole(AppRole appRole) throws RoleAlreadyExistsException, RoleFieldsInvalidException {
        if (!UserAccountAndRolesValidator.isValidRole(appRole)) {
            throw new RoleFieldsInvalidException();
        }
        AppRole formattedAppRole = UserAccountAndRolesValidator.formatRoles(appRole);

        if (getRoleByRoleName(formattedAppRole.getRoleName())!=null) {
            throw new RoleAlreadyExistsException();
        }
        return roleRepo.save(formattedAppRole);
    }

    @Override
    public AppRole getRoleByRoleName(String roleName) {
        AppRole appRole = roleRepo.findByRoleName(roleName);
        return appRole;
    }

    @Override
    public void addRoleToUser(String username, String roleName) throws AppRoleNotFoundException,
            UserAccountNotFoundException, UserAccountAlreadyAssignedThatRoleException {
        AppRole appRole = getRoleByRoleName(roleName);
        if(appRole==null){
            throw new AppRoleNotFoundException();
        }
       UserAccount userAccount =  getUserByUsername(username);
        if(userAccount==null){
            throw new UserAccountNotFoundException();
        }

        if(userAccount.getAppRoles().contains(appRole)){
            throw new UserAccountAlreadyAssignedThatRoleException();
        }
        userAccount.getAppRoles().add(appRole);

        new UserAccountRoleForm(username, roleName);
    }
}
